use pod::api_model::CreateItem;
use pod::command_line_interface::CliOptions;
use pod::database_api;
use pod::internal_api::{create_edge, create_item_tx};
use pod::plugin_auth_crypto::DatabaseKey;
use rusqlite::Transaction as Tx;
use serde_json::json;

pub fn prepare_graphql_dataset(
    tx: &Tx,
    pod_owner: &str,
    cli: CliOptions,
    database_key: DatabaseKey,
) {
    // insert age prop to Person
    let schema_json = json!({
        "type": "ItemPropertySchema",
        "itemType": "Person",
        "propertyName": "age",
        "valueType": "Integer",
    });
    let schema_struct: CreateItem = serde_json::from_value(schema_json).unwrap();
    let mut schema = database_api::get_schema(tx).unwrap();
    create_item_tx(tx, &mut schema, schema_struct, "", &cli, &database_key).unwrap();

    let mut schema = database_api::get_schema(tx).unwrap();
    // create level 1
    for i in 0..1000 {
        let person = json!({
            "type": "Person1",
            "id": "00000".to_owned() + &i.to_string(),
            "age": i
        });
        let person: CreateItem = serde_json::from_value(person).unwrap();
        create_item_tx(tx, &mut schema, person, pod_owner, &cli, &database_key).unwrap();
    }

    // create level 2
    for i in 1000..2000 {
        let person = json!({
            "type": "Person2",
            "id": "00000".to_owned() + &i.to_string(),
            "age": i
        });
        let person: CreateItem = serde_json::from_value(person).unwrap();
        create_item_tx(tx, &mut schema, person, pod_owner, &cli, &database_key).unwrap();
    }

    // create level 3
    for i in 2000..12000 {
        let person = json!({
            "type": "Person3",
            "id": "00000".to_owned() + &i.to_string()
        });
        let person: CreateItem = serde_json::from_value(person).unwrap();
        create_item_tx(tx, &mut schema, person, pod_owner, &cli, &database_key).unwrap();
    }

    // create schema for connections
    let schema_json = json!({
        "type": "ItemEdgeSchema",
        "edgeName": "friend1",
        "sourceType": "Person1",
        "targetType": "Person2",
    });
    let schema_struct: CreateItem = serde_json::from_value(schema_json).unwrap();
    let result = create_item_tx(
        tx,
        &mut database_api::get_schema(tx).unwrap(),
        schema_struct,
        pod_owner,
        &cli,
        &database_key,
    );
    assert!(result.is_ok());

    let schema_json = json!({
        "type": "ItemEdgeSchema",
        "edgeName": "friend2",
        "sourceType": "Person2",
        "targetType": "Person3",
    });
    let schema_struct: CreateItem = serde_json::from_value(schema_json).unwrap();
    let result = create_item_tx(
        tx,
        &mut database_api::get_schema(tx).unwrap(),
        schema_struct,
        pod_owner,
        &cli,
        &database_key,
    );
    assert!(result.is_ok());

    // Get updated schema
    let mut schema = database_api::get_schema(tx).unwrap();

    // create edges level 1 -> level 2
    for i in 0..100 {
        let _edge = {
            let json = json!({
                "_source": "00000".to_owned() + &i.to_string(),
                "_target": "00000".to_owned() + &(i+1000).to_string(),
                "_name": "friend1",
            });
            let parsed = serde_json::from_value(json).unwrap();
            create_edge(tx, &mut schema, parsed).unwrap()
        };
    }
    // create edges level 2 -> level 3
    for i in 0..1000 {
        let _edge = {
            let json = json!({
                "_source": "00000".to_owned() + &((i/10)+1000).to_string(),
                "_target": "00000".to_owned() + &(i+2000).to_string(),
                "_name": "friend2",
            });
            let parsed = serde_json::from_value(json).unwrap();
            create_edge(tx, &mut schema, parsed).unwrap()
        };
    }
}
